@@ -31,7 +31,7 @@ tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx;
 extern int debug, numcores, offlinemode, foreground, verbose, stupidfs, forcemanaged, keepnver,
 verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize, usewrap,
 exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, dirperms, fileperms,
-maxtempdelay;
+maxtempdelay, redirmax;
 
 // processed config settings
 extern tHttpUrl proxy_info;
@@ -96,4 +96,5 @@ bool MatchUncacheableTarget(const mstring &);
 bool CompileExpressions();
 }
 
+
 #endif
