@@ -23,7 +23,7 @@ struct foo;
 #define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
 #define SABSPATH(x) (CACHE_BASE+(x))
 
-#define REDIRMAX 5
+#define REDIRMAX acfg::redirmax
 
 #define FAKEDATEMARK "Sat, 26 Apr 1986 01:23:39 GMT+3"
 
