@@ -35,7 +35,7 @@ class CAddrInfo : public lockable
 {
 public:
 	time_t m_nExpTime;
-	struct addrinfo * m_bestInfo;
+	struct addrinfo * m_addrInfo;
 	
 	CAddrInfo();
 	bool Resolve(const mstring & sHostname, const mstring &sPort, mstring & sErrorBuf);
