@@ -10,8 +10,7 @@
 //#include <netinet/in.h>
 //#include <netdb.h>
 
-//#include "tcpconnect.h"
-
+#include "tcpconnect.h"
 #include "lockable.h"
 #include "fileitem.h"
 #include "acfg.h"
@@ -64,7 +63,7 @@ class dlcon : public lockable
 
     	tSS m_sendBuf, m_inBuf;
 
-    	UINT ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive);
+    	UINT ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &qActive);
 
 };
 
