@@ -46,6 +46,7 @@ class header {
       ~header();
       
       static mstring GenInfoHeaders();
+      static bool ParseDate(const char *, struct tm*);
 
       /*!
        * Read buffer to parse one string. Optional offset where to begin to
@@ -79,7 +80,7 @@ class header {
 inline bool BODYFREECODE(int status)
 {
 	// no response if not-modified or similar, following http://www.w3.org/Protocols/rfc2616/rfc2616-sec4.html#sec4.4
-	return (304 == (status) || ((status)>=100&&(status)<200) || 204==(status));
+	return (304 == status || (status>=100 && status<200) || 204==status);
 }
 
 
