@@ -72,19 +72,14 @@ public:
 	private:
 		tWUIPage &m_parent;
 		tFmtSendObj operator=(const tWUIPage::tFmtSendObj&);
-
-		// shall be copied only once when passing it to the stack
-		friend class tWUIPage;
-		inline tFmtSendObj(const tWUIPage::tFmtSendObj &x) : m_parent(x.m_parent) {}
 	};
 
-	inline tFmtSendObj SendFmt() { return tFmtSendObj(*this); }
+#define SendFmt() tFmtSendObj(*this)
 	char *m_fmtBuf;
 	size_t m_fmtSize;
 
 };
 
-
-void DispatchAndRunMaintTask(mstring &, int, const char*);
+void DispatchAndRunMaintTask(cmstring &cmd, int fd, const char *auth);
 
 #endif /*MAINTENANCE_H_*/
