@@ -44,6 +44,14 @@ protected:
 	tHttpUrl * m_proxy;
 
 	acfg::tRepoData::IHookHandler *m_pConnStateObserver;
+
+	mstring m_lastFileKeyPath;
+
+public:
+	//! @brief Remember the file name belonging to the recently initiated transfer
+	inline void NameLastFile(cmstring& keypath) { m_lastFileKeyPath=keypath; }
+	//! @brief Invalidate (truncate) recently touched file
+	void KillLastFile();
 };
 
 
