@@ -5,7 +5,6 @@
 #include "acfg.h"
 #include "dlcon.h"
 
-#include "tcpconnect.h"
 #include "fileitem.h"
 #include "fileio.h"
 #include "sockio.h"
@@ -45,6 +44,7 @@ struct tDlJob
 #define EFLAG_STORE_COLLISION 16
 #define EFLAG_HINT_SWITCH 32
 #define EFLAG_LOST_CON 64
+#define EFLAG_KILL_LAST_FILE 128
 
 	inline const mstring GetPeerName()
 	{
@@ -64,7 +64,9 @@ struct tDlJob
 
 	typedef enum
 	{
-		STATE_GETHEADER, STATE_GETDATA, STATE_GETDATA_CHUNKED, STATE_GETCHUNKHEAD, STATE_FINISHJOB
+		STATE_GETHEADER, STATE_PROCESS_DATA,
+		STATE_GETCHUNKHEAD, STATE_PROCESS_CHUNKDATA, STATE_GET_CHUNKTRAILER,
+		STATE_FINISHJOB
 	} tDlState;
 
 	const acfg::tRepoData * m_pBEdata;
@@ -246,7 +248,7 @@ struct tDlJob
 
 		if (m_nRest == 0)
 		{
-			m_DlState = (STATE_GETDATA == m_DlState) ? STATE_FINISHJOB : STATE_GETCHUNKHEAD;
+			m_DlState = (STATE_PROCESS_DATA == m_DlState) ? STATE_FINISHJOB : STATE_GETCHUNKHEAD;
 		}
 		else
 			return EFLAG_MORE; // will come back
@@ -256,7 +258,7 @@ struct tDlJob
 
 	/*!
 	 *
-	 * Process incoming traffic and write it down to disk/downloaders.
+	 * Process new incoming data and write it down to disk or other receivers.
 	 */
 	uint_fast8_t ProcessIncomming(acbuf & inBuf)
 	{
@@ -264,24 +266,28 @@ struct tDlJob
 		if (!m_pStorage)
 			return EFLAG_DISCON | EFLAG_JOB_BROKEN;
 
-		Reswitch:
-		ldbg("switch: " << m_DlState);
-
-		switch (m_DlState)
+		for (;;) // returned by explicit error (or get-more) return
 		{
+			ldbg("switch: " << m_DlState);
 
-		case (STATE_GETHEADER):
-		{
-			ldbg("STATE_GETHEADER");
-			header h;
-			if (inBuf.size() == 0)
-				return EFLAG_MORE;
-			dbgline;
-			int l = h.LoadFromBuf(inBuf.rptr(), inBuf.size());
-			if (0 == l)
-				return EFLAG_MORE;
-			else if (l > 0)
+			if ( STATE_GETHEADER == m_DlState)
 			{
+				ldbg("STATE_GETHEADER");
+				header h;
+				if (inBuf.size() == 0)
+					return EFLAG_MORE;
+				dbgline;
+				int l = h.LoadFromBuf(inBuf.rptr(), inBuf.size());
+				if (0 == l)
+					return EFLAG_MORE;
+				if (l<0)
+				{
+					dbgline;
+					sErrorMsg = "500 Invalid header";
+					// can be followed by any junk... drop that mirror, previous file could also contain bad data
+					return EFLAG_MIRROR_BROKEN | EFLAG_DISCON | EFLAG_KILL_LAST_FILE;
+				}
+
 				ldbg("contents: " << MYSTD::string(inBuf.rptr(), l));
 				inBuf.drop(l);
 				if (h.type != header::ANSWER)
@@ -290,145 +296,122 @@ struct tDlJob
 					// smells fatal...
 					return EFLAG_MIRROR_BROKEN | EFLAG_DISCON;
 				}
-			}
-			else
-			{
-				dbgline;
-				sErrorMsg = "500 Invalid header";
-				// can be followed by any junk... drop that mirror
-				return EFLAG_MIRROR_BROKEN | EFLAG_DISCON;
-			}
+				ldbg("GOT, parsed: " << h.frontLine);
 
-			ldbg("GOT, parsed: " << h.frontLine);
-
-			// explicitly blacklist mirror if key file is missing
-			if (h.getStatus() >= 400 && m_pBEdata)
-			{
-				for (tStrVecIterConst it = m_pBEdata->m_keyfiles.begin();
-						it != m_pBEdata->m_keyfiles.end(); ++it)
-						{
-					if (endsWith(m_fileUri.sPath, *it))
+				// explicitly blacklist mirror if key file is missing
+				if (h.getStatus() >= 400 && m_pBEdata)
+				{
+					for (tStrVecIterConst it = m_pBEdata->m_keyfiles.begin();
+							it != m_pBEdata->m_keyfiles.end(); ++it)
 					{
-						sErrorMsg = "500 Bad mirror";
-						return EFLAG_DISCON | EFLAG_MIRROR_BROKEN;
+						if (endsWith(m_fileUri.sPath, *it))
+						{
+							sErrorMsg = "500 Bad mirror";
+							return EFLAG_DISCON | EFLAG_MIRROR_BROKEN;
+						}
 					}
 				}
-			}
 
-			const char *p =
-					h.h[header::CONNECTION] ?
-							h.h[header::CONNECTION] :
-							h.h[header::PROXY_CONNECTION];
-			if (p && 0 == strcasecmp(p, "close"))
-			{
-				ldbg("Peer wants to close connection after request");
-				m_eReconnectASAP = EFLAG_DISCON;
-			}
-			if (m_pStorage->m_bHeadOnly)
-				m_DlState = STATE_FINISHJOB;
-			else if (h.h[header::TRANSFER_ENCODING] &&
-					0 == strcasecmp(h.h[header::TRANSFER_ENCODING], "chunked"))
-			{
-				m_DlState = STATE_GETCHUNKHEAD;
-				h.del(header::TRANSFER_ENCODING); // don't care anymore
-			}
-			else
-			{
-				dbgline;
-				p = h.h[header::CONTENT_LENGTH];
-				if (!p)
+				const char *pCon = h.h[header::CONNECTION];
+				if(!pCon)
+					pCon = h.h[header::PROXY_CONNECTION];
+				if (pCon && 0 == strcasecmp(pCon, "close"))
 				{
-					sErrorMsg = "500 Missing Content-Length";
-					return EFLAG_DISCON | EFLAG_JOB_BROKEN;
+					ldbg("Peer wants to close connection after request");
+					m_eReconnectASAP = EFLAG_DISCON;
+				}
+
+				if (m_pStorage->m_bHeadOnly)
+					m_DlState = STATE_FINISHJOB;
+				else if (h.h[header::TRANSFER_ENCODING]
+						&& 0 == strcasecmp(h.h[header::TRANSFER_ENCODING], "chunked"))
+				{
+					m_DlState = STATE_GETCHUNKHEAD;
+					h.del(header::TRANSFER_ENCODING); // don't care anymore
+				}
+				else
+				{
+					dbgline;
+					if (!h.h[header::CONTENT_LENGTH])
+					{
+						sErrorMsg = "500 Missing Content-Length";
+						return EFLAG_DISCON | EFLAG_JOB_BROKEN;
+					}
+					// may support such endless stuff in the future but that's too unreliable for now
+					m_nRest = atoofft(h.h[header::CONTENT_LENGTH]);
+					m_DlState = STATE_PROCESS_DATA;
+				}
+				h.set(header::XORIG, RemoteUri());
+				if (!m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr()))
+				{
+					ldbg("Item dl'ed by others or in error state --> drop it, reconnect");
+					m_DlState = STATE_PROCESS_DATA;
+					return EFLAG_DISCON | EFLAG_JOB_BROKEN | EFLAG_STORE_COLLISION;
 				}
-				// may support such endless stuff in the future but that's too unreliable for now
-				m_nRest = atoofft(p);
-				m_DlState = STATE_GETDATA;
 			}
-
-			h.set(header::XORIG, RemoteUri());
-
-			if (!m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr()))
+			else if (m_DlState == STATE_PROCESS_CHUNKDATA || m_DlState ==  STATE_PROCESS_DATA)
 			{
-				ldbg("Item dl'ed by others or in error state --> drop it, reconnect");
-				m_DlState = STATE_GETDATA;
-				return EFLAG_DISCON | EFLAG_JOB_BROKEN | EFLAG_STORE_COLLISION;
+				// similar states, just handled differently afterwards
+				ldbg("STATE_GETDATA");
+				uint_fast8_t res = NewDataHandler(inBuf);
+				if (EFLAG_HINT_SWITCH != res)
+					return res;
 			}
-			goto Reswitch;
-
-		}
-		case (STATE_GETDATA_CHUNKED): // fall through, just send it back to header parser hereafter
-			ldbg("STATE_GETDATA_CHUNKED (to STATE_GETDATA)");
-		case (STATE_GETDATA):
-		{
-			ldbg("STATE_GETDATA");
-			uint_fast8_t res = NewDataHandler(inBuf);
-			if (EFLAG_HINT_SWITCH != res)
-				return res;
-			goto Reswitch;
-		}
-		case (STATE_FINISHJOB):
-		{
-			ldbg("STATE_FINISHJOB");
-			m_DlState = STATE_GETHEADER;
-			m_pStorage->StoreFileData(NULL, 0);
-			return EFLAG_DONE | m_eReconnectASAP;
-
-		}
-		case (STATE_GETCHUNKHEAD):
-		{
-			ldbg("STATE_GETCHUNKHEAD");
-			const char *p = inBuf.c_str();
-			const char *e = strstr(p, "\r\n");
-			if (e == p)
-			{ // came back from reading, drop remaining junk?
-				inBuf.drop(2);
-				p += 2;
-				e = strstr(p, "\r\n");
-			}
-			dbgline;
-			if (!e)
+			else if (m_DlState == STATE_FINISHJOB)
 			{
-				inBuf.move();
-				return EFLAG_MORE; // get more data
+				ldbg("STATE_FINISHJOB");
+				m_DlState = STATE_GETHEADER;
+				m_pStorage->StoreFileData(NULL, 0);
+				return EFLAG_DONE | m_eReconnectASAP;
 			}
-			unsigned int len(0);
-			int n = sscanf(p, "%x", &len);
-
-			unsigned int nCheadSize = e - p + 2;
-			if (n == 1 && len > 0)
+			else if (m_DlState == STATE_GETCHUNKHEAD)
 			{
-				ldbg("ok, skip " << nCheadSize <<" bytes, " <<p);
-				inBuf.drop(nCheadSize);
-				m_nRest = len;
-				m_DlState = STATE_GETDATA_CHUNKED;
-			}
-			else if (n == 1)
-			{
-				// skip the additional \r\n of the null-sized part here as well
-				ldbg(
-						"looks like the end, but needs to get everything into buffer to change the state reliably");
-				if (inBuf.size() < nCheadSize + 2)
+				ldbg("STATE_GETCHUNKHEAD");
+				// came back from reading, drop remaining newlines?
+				while (inBuf.size() > 0)
+				{
+					char c = *(inBuf.rptr());
+					if (c != '\r' && c != '\n')
+						break;
+					inBuf.drop(1);
+				}
+				const char *crlf(0), *pStart(inBuf.c_str());
+				if (!inBuf.size() || NULL == (crlf = strstr(pStart, "\r\n")))
 				{
 					inBuf.move();
 					return EFLAG_MORE;
 				}
-				if (!(e[2] == '\r' && e[3] == '\n'))
+				UINT len(0);
+				if (1 != sscanf(pStart, "%x", &len))
+					return EFLAG_JOB_BROKEN; // hm...?
+				inBuf.drop(crlf + 2 - pStart);
+				if (len > 0)
 				{
-					aclog::err(m_pStorage->m_sPathRel + " -- error in chunk format detected");
-					return EFLAG_JOB_BROKEN;
+					m_nRest = len;
+					m_DlState = STATE_PROCESS_CHUNKDATA;
 				}
-
-				inBuf.drop(nCheadSize + 2);
-				m_DlState = STATE_FINISHJOB;
+				else
+					m_DlState = STATE_GET_CHUNKTRAILER;
 			}
-			else
-				return EFLAG_JOB_BROKEN; // that's bad...
-			goto Reswitch;
-			break;
-		}
+			else if (m_DlState == STATE_GET_CHUNKTRAILER)
+			{
+				if (inBuf.size() < 2)
+					return EFLAG_MORE;
+				const char *pStart(inBuf.c_str());
+				const char *crlf(strstr(pStart, "\r\n"));
+				if (!crlf)
+					return EFLAG_MORE;
 
+				if (crlf == pStart) // valid empty line -> done here
+				{
+					inBuf.drop(2);
+					m_DlState = STATE_FINISHJOB;
+				}
+				else
+					inBuf.drop(crlf + 2 - pStart); // drop line and watch for others
+			}
 		}
+		ASSERT(!"unreachable");
 		return EFLAG_JOB_BROKEN;
 	}
 
@@ -539,7 +522,7 @@ dlcon::~dlcon()
 	checkforceclose(m_wakepipe[1]);
 }
 
-inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
+inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &qActive)
 {
 	LOGSTART2("dlcon::ExchangeData",
 			"qsize: " << qActive.size() << ", sendbuf size: "
@@ -548,6 +531,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 	fd_set rfds, wfds;
 	struct timeval tv;
 	int r = 0;
+	int fd = con ? con->GetFD() : -1;
 	FD_ZERO(&rfds);
 	FD_ZERO(&wfds);
 
@@ -679,6 +663,9 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 
 				if (EFLAG_DONE & res)
 				{
+					// just in case that server damaged the last response body
+					con->NameLastFile(qActive.front()->m_pStorage->m_sPathRel);
+
 					qActive.pop_front();
 					if (EFLAG_DISCON & res)
 						return EFLAG_DISCON; // with cleaned flags
@@ -698,6 +685,10 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 				}
 				else // error handling, pass to main loop
 				{
+
+					if(EFLAG_KILL_LAST_FILE & res)
+						con->KillLastFile();
+
 					setIfNotEmpty(sErrorMsg, qActive.front()->sErrorMsg);
 					return res;
 				}
@@ -872,7 +863,7 @@ void dlcon::WorkLoop()
 		ASSERT( !inpipe.empty() || !bStopRequesting);
 
          // inner loop: plain communication until something happens. Maybe should use epoll here?
-        loopRes=ExchangeData(sErrorMsg, con ? con->GetFD() : -1, inpipe);
+        loopRes=ExchangeData(sErrorMsg, con, inpipe);
         ldbg("loopRes: "<< loopRes);
 
         // no matter what happened, that stop flag is now irrelevant
