@@ -210,10 +210,8 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	else // unknown... ok, probe it
 	{
 		filereader fh;
-		if (fh.OpenFile(sFilename, true))
+		if (fh.OpenFile(sFilename, true) && fh.GetSize() >= 10)
 		{
-			if (fh.GetSize() < 10)
-				return false;
 			if (0 == memcmp(gzMagic, fh.GetBuffer(), _countof(gzMagic)))
 				m_Dec.reset(new tGzDec);
 			else if (0 == memcmp(bz2Magic, fh.GetBuffer(), _countof(bz2Magic)))
@@ -222,7 +220,6 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 				m_Dec.reset(new tXzDec);
 			else if (0 == memcmp(lzmaMagic, fh.GetBuffer(), _countof(lzmaMagic)))
 				m_Dec.reset(new tXzDec(true));
-
 		}
 	}
 
