@@ -17,29 +17,50 @@
 #include "fileio.h"
 #include "filereader.h"
 
+#include <map>
+
 using namespace MYSTD;
 
-struct {
+#if 1
+struct eHeadPos2label
+{
 	header::eHeadPos pos;
 	const char *str;
-	}
-static mapId2Headname[] =
+};
+
+eHeadPos2label mapId2Headname[] =
 {
-		{ header::CONNECTION, "Connection"},
+		{ header::LAST_MODIFIED, "Last-Modified"},
 		{ header::CONTENT_LENGTH, "Content-Length"},
+		{ header::CONNECTION, "Connection"},
+		{ header::CONTENT_TYPE, "Content-Type"},
 		{ header::IF_MODIFIED_SINCE, "If-Modified-Since"},
 		{ header::RANGE, "Range"},
 		{ header::IFRANGE, "If-Range"},
 		{ header::CONTENT_RANGE, "Content-Range"},
-		{ header::LAST_MODIFIED, "Last-Modified"},
 		{ header::PROXY_CONNECTION, "Proxy-Connection"},
 		{ header::TRANSFER_ENCODING, "Transfer-Encoding"},
-		{ header::XORIG, "X-Original-Source"},
 		{ header::AUTHORIZATION, "Authorization"},
-		{ header::XFORWARDEDFOR, "X-Forwarded-For"},
 		{ header::LOCATION, "Location" },
-    { header::CONTENT_TYPE, "Content-Type" }
+		{ header::XFORWARDEDFOR, "X-Forwarded-For"},
+		{ header::XORIG, "X-Original-Source"}
 };
+#endif
+#if 0 // nonsense... save a penny, waste an hour
+struct tHeadLabelMap
+{
+	class noCaseComp
+	{
+	//	bool operator<(const tStringRef &a) { return strncasecmp(a.first, first, second)<0; }
+	};
+	map<pair<const char*,size_t>, header::eHeadPos> lookup;
+	tHeadLabelMap()
+	{
+		//tHeadLabelMap &x=*this;
+		insert(make_pair(tStringRef(NAMEWLEN("foo")), header::XORIG));
+	}
+} label_map;
+#endif
 
 header::header()
 :
@@ -63,7 +84,7 @@ header& header::operator=(const header& s)
 	type=s.type;
 	frontLine=s.frontLine;
 	m_nEstimLength=s.m_nEstimLength;
-	for (UINT i = 0; i < HEADPOS_MAX; i++)
+	for (UINT i = 0; i < HEADPOS_MAX; ++i)
 	{
 		if (h[i])
 			free(h[i]);
@@ -75,10 +96,7 @@ header& header::operator=(const header& s)
 header::~header()
 {
 	for(UINT i=0; i<HEADPOS_MAX; i++)
-	{
 		free(h[i]);
-		h[i]=NULL;
-	}
 }
 
 void header::clear()
@@ -152,34 +170,24 @@ inline int header::Load(const char * const in, UINT maxlen)
 		if (!sep)
 			return -1;
 		
-		// use as cut buffer
-		string key;
-		key.assign(szBegin, sep-szBegin);
+		const char *key = szBegin;
+		size_t keyLen=sep-szBegin;
 
-		// trimFront would be slower...
 		sep++;
 		while (sep<end && isspace((UINT)*sep))
 			sep++;
 		
-		for(UINT n=0; n<_countof(mapId2Headname); n++)
+		for(const eHeadPos2label *pMap=mapId2Headname;
+				pMap<mapId2Headname+_countof(mapId2Headname); ++pMap)
 		{
-			eHeadPos idx = mapId2Headname[n].pos;
-			//const char *szName=szHeadNames[i];
-			if (0==strcasecmp(mapId2Headname[n].str, key.c_str()))
-			{
-				if (h[idx])
-				{
-					free(h[idx]);
-					h[idx]=NULL;
-				}
-				UINT l=end-sep;
-				h[idx] = (char *) malloc(l+1);
-				if (h[idx])
-				{
-					memcpy(h[idx], sep, l);
-					h[idx][l]='\0';
-				}
-			}
+			if (strncasecmp(pMap->str, key, keyLen))
+				continue;
+			UINT l=end-sep;
+			if( ! (h[pMap->pos] = (char*) realloc(h[pMap->pos], l+1)))
+				continue;
+			memcpy(h[pMap->pos], sep, l);
+			h[pMap->pos][l]='\0';
+			break;
 		}
 	}
 	return -2;
@@ -222,13 +230,13 @@ void header::set(eHeadPos i, const char *val)
 
 void header::set(eHeadPos i, const char *val, size_t len)
 {
-	if (h[i])
+	if(!val)
 	{
 		free(h[i]);
 		h[i]=NULL;
+		return;
 	}
-	if(val)
-		h[i] = (char*) malloc(len+1);
+	h[i] = (char*) realloc(h[i], len+1);
 	if(h[i])
 	{
 		memcpy(h[i], val, len);
@@ -236,7 +244,8 @@ void header::set(eHeadPos i, const char *val, size_t len)
 	}
 }
 
-void header::set(eHeadPos key, const MYSTD::string &value) {
+void header::set(eHeadPos key, cmstring &value)
+{
 	string::size_type l=value.size()+1;
 	h[key]=(char*) realloc(h[key], l);
 	if(h[key])
@@ -246,25 +255,25 @@ void header::set(eHeadPos key, const MYSTD::string &value) {
 void header::set(eHeadPos key, off_t nValue)
 {	
 	char buf[3*sizeof(off_t)];
-	sprintf(buf, OFF_T_FMT, nValue);
-    set(key, buf);
+	int len=sprintf(buf, OFF_T_FMT, nValue);
+    set(key, buf, len);
 }
 
 #ifndef MINIBUILD
 
 mstring header::ToString() const
 {
-
-	string hstr = frontLine + "\r\n";
-	for (UINT n = 0; n < _countof(mapId2Headname); n++)
+	string s = frontLine;
+	s+="\r\n";
+	for(const eHeadPos2label *pMap=mapId2Headname;
+			pMap<mapId2Headname+_countof(mapId2Headname); ++pMap)
 	{
-		eHeadPos idx = mapId2Headname[n].pos;
-		if (h[idx])
+		if (h[pMap->pos])
 		{
-			hstr.append(mapId2Headname[n].str);
-			hstr += ": ";
-			hstr += h[idx];
-			hstr += "\r\n";
+			s += pMap->str;
+			s += ": ";
+			s += h[pMap->pos];
+			s += "\r\n";
 		}
 	}
 	char buf[26];
@@ -272,10 +281,10 @@ mstring header::ToString() const
 	const time_t cur = time(NULL);
 	gmtime_r(&cur, &tmp);
 	asctime_r(&tmp, buf);
-	hstr += "Date: ";
-	hstr.append(buf, 24);
-	hstr += "\r\n\r\n";
-	return hstr;
+	s+="Date: ";
+	s.append(buf, 24);
+	s+= "\r\n\r\n";
+	return s;
 }
 
 int header::StoreToFile(const string &sPath) const
