@@ -494,7 +494,7 @@ inline bool job::ParseRange()
 bool job::CreateRedirect(header & respHead)
 {
 	int st = respHead.getStatus();
-	if (m_nRedirRemaining <= 0 || (st != 301 && st != 302) // redir limit reached or not redirect
+	if (m_nRedirRemaining <= 0 || (st != 301 && st != 302 && st != 307) // redir limit reached or not redirect
 	|| !respHead.h[header::LOCATION] || !respHead.h[header::LOCATION][0]) // or nonsense
 		return false;
 
