@@ -67,7 +67,7 @@ void tWUIPage::SendChunk(const char *data, size_t len)
 
 	if(m_reportFD<3)
 	{
-		write(m_reportFD, data, len);
+		ignore_value(::write(m_reportFD, data, len));
 		return;
 	}
 
