@@ -89,7 +89,7 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 			/*	if ( (IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_SHA1, true))
 			 || (!IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_MD5, false)))
 			 */
-			if (!fpr.ScanFile(sPath, ctp, false))
+			if (!fpr.ScanFile(sPath, ctp, false, NULL))
 			{
 				SendFmt() << "<font color=red>Error checking " << sPath << "</font><br>\n";
 				continue;
