@@ -15,7 +15,7 @@
 #include "caddrinfo.h"
 #include <signal.h>
 #include "fileio.h"
-
+#include "fileitem.h"
 #include "cleaner.h"
 
 using namespace MYSTD;
@@ -167,6 +167,7 @@ void tcpconnect::Disconnect()
 		m_pConnStateObserver->JobRelease();
 
 	m_pConnStateObserver=NULL;
+	m_lastFileKeyPath.clear();
 
 	forceShutdownClose(m_conFd);
 }
@@ -270,7 +271,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	}
 
 #ifndef NOCONCACHE
-	time_t now(time(0));
+	time_t now=GetTime();
 	lockguard __g(conPoolMx);
 
 	// a DOS?
@@ -290,7 +291,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 time_t tcpconnect::ExpireCache()
 {
 	lockguard __g(conPoolMx);
-	time_t now(time(0));
+	time_t now=GetTime();
 
 	fd_set rfds;
 	FD_ZERO(&rfds);
@@ -323,5 +324,21 @@ time_t tcpconnect::ExpireCache()
 			++it;
 	}
 
-	return spareConPool.empty() ? cleaner::never : time(0)+TIME_SOCKET_EXPIRE_CLOSE/4+1;
+	return spareConPool.empty() ? cleaner::never : GetTime()+TIME_SOCKET_EXPIRE_CLOSE/4+1;
 }
+
+void tcpconnect::KillLastFile()
+{
+#ifndef MINIBUILD
+	if (m_lastFileKeyPath.empty())
+		return;
+
+	tFileItemPtr p(fileitem::GetRegisteredFileItem(m_lastFileKeyPath));
+	if (!p)
+		return;
+
+	p->SetupClean(true);
+	p->Unreg();
+#endif
+}
+
